//
//  ExtendedLocation.m
//  GPS Master
//
//  Created by Eric Birgel on 19.03.12.
//  Copyright (c) 2012 __MyCompanyName__. All rights reserved.
//

#import "ExtendedLocation.h"
#import "Base64.h"

@implementation ExtendedLocation

@synthesize CurrentLocation;

-(id)initWithLocation:(CLLocation*)Location {
    
    if(self = [self init]) {
        CurrentLocation = Location;
    }
    
    return self;
}
-(double)getSpeed:(NSString *)Unit{
    //Get Speed
    double speed = -1;
    double kmh = ([CurrentLocation speed]*60*60)/1000;
    double mph = kmh/1.609344;
    double knot = kmh/1.851999999999052;
    
    if([Unit isEqualToString:@"mph"])
        speed = ([CurrentLocation speed] >= 0) ? mph : 0;
    if([Unit isEqualToString:@"kmh"])
        speed = ([CurrentLocation speed] >= 0) ? kmh : 0;
    if([Unit isEqualToString:@"knot"])
        speed = ([CurrentLocation speed] >= 0) ? knot : 0;
    
    return speed;
}
-(int)getAltitude:(NSString *)Unit{
    //Get Speed
    double altitude = -1;
    double m = [CurrentLocation altitude];
    double ft = m*3.2808399;
    
    if([Unit isEqualToString:@"m"])
        altitude = m;
    if([Unit isEqualToString:@"ft"])
        altitude = ft;
    
    return (int)altitude;
}
-(int)getSignalStrenght:(NSString *)Type{
    double signalStrength = ([CurrentLocation horizontalAccuracy]+[CurrentLocation verticalAccuracy])/2;
    
    if ([Type isEqualToString:@"percent"]) {
        if(signalStrength < 1000)
        {
            signalStrength = 100-(((log2(signalStrength) - 2.32)/7.68)*100);
            signalStrength = (signalStrength>=100) ? 100 : signalStrength;
            return (int)signalStrength;
        }
        else
        {
            return 0;
        }
    }
    if ([Type isEqualToString:@"ft"]) {
        signalStrength = signalStrength*3.2808399;
        return signalStrength;
    }
    
    return signalStrength;
}
-(LocationCoordinate2DConverted)getLocationwithFormat:(NSString *)Format{
    CLLocationCoordinate2D coordinate = [CurrentLocation coordinate];
    LocationCoordinate2DConverted LocationwithFormat;
    
    //Convert
    LocationwithFormat.latitudeDMS = [self degToDms:coordinate.latitude];
    LocationwithFormat.longitudeDMS = [self degToDms:coordinate.longitude];
    LocationwithFormat.latitudeWithFormat = @"error with format";
    LocationwithFormat.longitudeWithFormat = @"error with format";
    
    //Format it like this: N 47° 37' 58.512" E 7° 48' 55.296"
    //The degree can be positive (North(latitude) and East(longitude)) or negative (South and West).
    if([Format isEqualToString:@"DMS"])
    {
        NSString *lat = LocationwithFormat.latitudeDMS.positive == FALSE ? @"S" : @"N";
        LocationwithFormat.latitudeWithFormat = [NSString stringWithFormat:@"%@ %.f° %.f' %.04f",
                                                lat,
                                                LocationwithFormat.latitudeDMS.degrees,
                                                LocationwithFormat.latitudeDMS.minutes,
                                                LocationwithFormat.latitudeDMS.seconds
                                                ];
        
        NSString *lon = LocationwithFormat.longitudeDMS.positive == FALSE ? @"W" : @"E";
        LocationwithFormat.longitudeWithFormat = [NSString stringWithFormat:@"%@ %.f° %.f' %.04f",
                                                 lon,
                                                 LocationwithFormat.longitudeDMS.degrees,
                                                 LocationwithFormat.longitudeDMS.minutes,
                                                 LocationwithFormat.longitudeDMS.seconds
                                                 ];
    }
    if([Format isEqualToString:@"MinDec"])
    {
        NSString *lat = LocationwithFormat.latitudeDMS.positive == FALSE ? @"S" : @"N";
        LocationwithFormat.latitudeWithFormat = [NSString stringWithFormat:@"%@ %.f° %.03f'",
                                                 lat,
                                                 LocationwithFormat.latitudeDMS.degrees,
                                                 LocationwithFormat.latitudeDMS.minutesDouble
                                                 ];
        
        NSString *lon = LocationwithFormat.longitudeDMS.positive == FALSE ? @"W" : @"E";
        LocationwithFormat.longitudeWithFormat = [NSString stringWithFormat:@"%@ %.f° %.03f'",
                                                  lon,
                                                  LocationwithFormat.longitudeDMS.degrees,
                                                  LocationwithFormat.longitudeDMS.minutesDouble
                                                  ];
    }
    if([Format isEqualToString:@"DegDec"])
    {
        NSString *lat = LocationwithFormat.latitudeDMS.positive == FALSE ? @"S" : @"N";
        CLLocationDegrees degLat = LocationwithFormat.latitudeDMS.positive == FALSE ? coordinate.latitude*-1 : coordinate.latitude;
        LocationwithFormat.latitudeWithFormat = [NSString stringWithFormat:@"%@ %.06f°",
                                                 lat,
                                                 degLat
                                                 ];
        
        NSString *lon = LocationwithFormat.longitudeDMS.positive == FALSE ? @"W" : @"E";
        CLLocationDegrees degLon = LocationwithFormat.longitudeDMS.positive == FALSE ? coordinate.longitude*-1 : coordinate.longitude;
        LocationwithFormat.longitudeWithFormat = [NSString stringWithFormat:@"%@ %.06f°",
                                                  lon,
                                                  degLon
                                                  ];
    }
    if([Format isEqualToString:@"Base64"])
    {
        NSNumber *convert1 = [NSNumber numberWithDouble:coordinate.latitude];
        float floatLat =  [convert1 floatValue];
        NSNumber *convert2 = [NSNumber numberWithDouble:coordinate.longitude];
        float floatLon = [convert2 floatValue];
        NSData * lat = [NSData dataWithBytes:&floatLat  length:sizeof(float)];
        NSData * lon = [NSData dataWithBytes:&floatLon length:sizeof(float)];
        LocationwithFormat.latitudeWithFormat = [[lat base64String] stringByReplacingOccurrencesOfString:@"=" withString:@""];
        LocationwithFormat.longitudeWithFormat = [[lon base64String] stringByReplacingOccurrencesOfString:@"=" withString:@""];
    }

    
    return LocationwithFormat;
}
-(LocationCoordinateInDMS)degToDms:(CLLocationDegrees)Location{
    LocationCoordinateInDMS ConvertedLocation;
    ConvertedLocation.positive = TRUE;
    
    if(Location < 0)
    {
        Location = Location*-1;
        ConvertedLocation.positive = FALSE;
    }
    
    int deg = (int)Location;
    double  minFloat = 60*(Location-deg);
    int min = (int)minFloat;
    double  secFloat = 60*(minFloat-min);
    
    //Round seconds to desired accuracy After rounding, the seconds might become 60
    secFloat = secFloat;
    
    if(secFloat >= 60)
    {
        min = min+1;
        secFloat = 0;
    }  
    if(min >= 60)
    {
        deg = deg+1;
        min = 0;
    }
    ConvertedLocation.degrees = deg;
    ConvertedLocation.minutes = min;
    ConvertedLocation.minutesDouble = minFloat;
    ConvertedLocation.seconds = secFloat;
    
    return ConvertedLocation;
}



@end
