//
//  ViewController.m
//  simplynorth
//
//  Created by Eric Birgel on 12.07.12.
//  Copyright (c) 2012 __MyCompanyName__. All rights reserved.
//

#import "ViewController.h"

#define radianConst M_PI/180.0

@interface ViewController (PrivateMethods)
- (void)loadBlackSkin;
- (void)openAbout;
- (void)updateGui;
- (void)updateGuiLocation;
- (void)loadUserSettings;
- (void)fadeInLabel;
- (void)tabAction:(UITapGestureRecognizer *)recognizer;
- (void)handleSwipeFrom:(UISwipeGestureRecognizer *)recognizer;
- (void)showHudonView:(NSString*)label WithDetailLabel:(NSString*)detailLabel andImage:(NSString*)image;
@end

@implementation ViewController



//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//----------------------  Initialisation and Memory Handling ----------------------------
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

- (void)viewDidLoad
{
    [super viewDidLoad];
    
    
	// Do any additional setup after loading the view, typically from a nib.
    pasteboard = [UIPasteboard generalPasteboard];
    CLController = [[CoreLocationController alloc] init];
	CLController.delegate = self;
    trueNorth = true;
    stopGuiUpdate = false;
    tempLocation = nil;
    viewWindrose.autoresizingMask = UIViewAutoresizingNone;
    iPadWindhosePortrait = viewWindrose.frame;
    iPadWindhoseLandscape = viewWindrose.frame;
    iPadWindhoseLandscape.origin.x = 254;
    
    
    actualDegrees = 0;
    locationFormatPointer = 0;
    smoothDegrees = [[NSMutableArray alloc] init];
    locationFormats = [[NSMutableArray alloc] init];
    [locationFormats addObject:[NSString stringWithFormat:@"MinDec"]];
    [locationFormats addObject:[NSString stringWithFormat:@"DMS"]];
    [locationFormats addObject:[NSString stringWithFormat:@"DegDec"]];
    [locationFormats addObject:[NSString stringWithFormat:@"Feature"]];
    
    [self loadUserSettings];
    
    
    timerUpdateGUI = [NSTimer scheduledTimerWithTimeInterval:0.025 target:self selector:@selector(updateGui) userInfo:nil repeats:YES];
    [timerUpdateGUI retain];
    
    //Swipes
    recognizerRight = [[UISwipeGestureRecognizer alloc] initWithTarget:self action:@selector(handleSwipeFrom:)];
    [recognizerRight setDirection:(UISwipeGestureRecognizerDirectionRight)];
    [[self view] addGestureRecognizer:recognizerRight];
    recognizerDown = [[UISwipeGestureRecognizer alloc] initWithTarget:self action:@selector(handleSwipeFrom:)];
    [recognizerDown setDirection:(UISwipeGestureRecognizerDirectionDown)];
    [[self view] addGestureRecognizer:recognizerDown];
    recognizerLeft = [[UISwipeGestureRecognizer alloc] initWithTarget:self action:@selector(handleSwipeFrom:)];
    [recognizerLeft setDirection:(UISwipeGestureRecognizerDirectionLeft)];
    [[self view] addGestureRecognizer:recognizerLeft];
    
    //Taps
    recognizerTapLong = [[[UITapGestureRecognizer alloc] initWithTarget:self action:@selector(tabAction:)] autorelease];
    [labelLong addGestureRecognizer:recognizerTapLong];
    recognizerTapLat = [[[UITapGestureRecognizer alloc] initWithTarget:self action:@selector(tabAction:)] autorelease];
    [labelLat addGestureRecognizer:recognizerTapLat];
    
    //Transparent Overlay View
    //self.modalPresentationStyle = UIModalPresentationCurrentContext;
}


- (void)viewWillAppear:(BOOL)animated
{
    [self updateGuiLocation];
    [CLController.locMgr startUpdatingLocation];
    [CLController.locMgr startUpdatingHeading];
}

- (void)viewDidAppear:(BOOL)animated
{
    int appearFirstDialog = [[NSUserDefaults standardUserDefaults] integerForKey:@"appearFirst"];
    if (appearFirstDialog != 1234) {
        [[NSUserDefaults standardUserDefaults] setInteger:1234 forKey:@"appearFirst"];
        [[NSUserDefaults standardUserDefaults] synchronize];
        [self openAbout];
    }
}

- (void)viewWillDisappear:(BOOL)animated 
{
    [CLController.locMgr stopUpdatingLocation];
    [CLController.locMgr stopUpdatingHeading];
}

- (void)viewDidUnload
{
    [recognizerDown release];
    recognizerDown = nil;
    [recognizerLeft release];
    recognizerLeft = nil;
    [recognizerRight release];
    recognizerRight = nil;
    [labelDegrees release];
    labelDegrees = nil;
    [viewWindrose release];
    viewWindrose = nil;
    [labelLat release];
    labelLat = nil;
    [labelLong release];
    labelLong = nil;
    [north release];
    north = nil;
    [imgMainBG release];
    imgMainBG = nil;
    [imgBigArrow release];
    imgBigArrow = nil;
    [imgCompassMain release];
    imgCompassMain = nil;
    [imgCompassWindrose release];
    imgCompassWindrose = nil;
    [imgCompassGlass release];
    imgCompassGlass = nil;
    [busybytes release];
    busybytes = nil;
    [super viewDidUnload];
    // Release any retained subviews of the main view.
}

- (void)loadUserSettings {
    NSUserDefaults* locationFormat = [[NSUserDefaults standardUserDefaults] objectForKey:@"LocationFormat"];
    locationFormatPointer = locationFormat != nil ?[[NSUserDefaults standardUserDefaults] integerForKey:@"LocationFormat"]:0;
    NSUserDefaults* northFormat = [[NSUserDefaults standardUserDefaults] objectForKey:@"NorthFormat"];
    trueNorth = northFormat != nil ?[[NSUserDefaults standardUserDefaults] boolForKey:@"NorthFormat"]:true;

    
    if(trueNorth)
        [north setImage:[UIImage imageNamed: @"northArrowRed.png"] forState:UIControlStateNormal];
    else
        [north setImage:[UIImage imageNamed: @"northArrowBlue.png"] forState:UIControlStateNormal];
    
    [[NSUserDefaults standardUserDefaults] synchronize];
}

- (void)loadBlackSkin {
    [imgCompassMain setImage:[UIImage imageNamed:@"compass_main_black.png"]];
    [imgCompassWindrose setImage:[UIImage imageNamed:@"compassWindhose_black.png"]];
    [imgCompassGlass setImage:[UIImage imageNamed:@"compass_glass_black.png"]];
    [imgMainBG setImage:[UIImage imageNamed:@"MainBG_black.png"]];
    [imgBigArrow setImage:[UIImage imageNamed:@"bigArrow_black.png"]];
    [busybytes setBackgroundImage:[UIImage imageNamed:@"busybytesLogo_black.png"] forState:UIControlStateNormal];
    
    
    [labelDegrees setTextColor:[UIColor colorWithRed:(169/255.f) green:(173/255.f) blue:(182/255.f) alpha:1.0]];
    //[labelDegrees setShadowColor:[UIColor colorWithRed:(169/255.f) green:(173/255.f) blue:(182/255.f) alpha:1.0]];
}

- (BOOL)shouldAutorotate {
    
    UIInterfaceOrientation interfaceOrientation = [[UIDevice currentDevice] orientation];
    
    deviceOrientation = interfaceOrientation;
    
    
    return  (interfaceOrientation == UIInterfaceOrientationPortrait) ||
    (interfaceOrientation == UIInterfaceOrientationPortraitUpsideDown);
}

- (BOOL)shouldAutorotateToInterfaceOrientation:(UIInterfaceOrientation)interfaceOrientation
{
    deviceOrientation = interfaceOrientation;
    
    
    return  (interfaceOrientation == UIInterfaceOrientationPortrait) ||
    (interfaceOrientation == UIInterfaceOrientationPortraitUpsideDown);
    
    if ([[UIDevice currentDevice] userInterfaceIdiom] == UIUserInterfaceIdiomPhone) {
        return  (interfaceOrientation == UIInterfaceOrientationPortrait) || 
        (interfaceOrientation == UIInterfaceOrientationPortraitUpsideDown);
    } else {
        
        if (interfaceOrientation == UIInterfaceOrientationLandscapeLeft || interfaceOrientation == UIInterfaceOrientationLandscapeRight) {
            viewWindrose.frame = iPadWindhoseLandscape;
                //
        }
        else {
            viewWindrose.frame = iPadWindhosePortrait;
                //
        }
        return  (interfaceOrientation == UIInterfaceOrientationPortrait) || 
        (interfaceOrientation == UIInterfaceOrientationPortraitUpsideDown) ||
        (interfaceOrientation == UIInterfaceOrientationLandscapeLeft) ||
        (interfaceOrientation == UIInterfaceOrientationLandscapeRight);
    }
}

- (void)willRotateToInterfaceOrientation:(UIInterfaceOrientation)toInterfaceOrientation duration:(NSTimeInterval)duration
{
//stopGuiUpdate = true;
}

- (void)didRotateFromInterfaceOrientation:(UIInterfaceOrientation)fromInterfaceOrientation
{
//stopGuiUpdate = false;
}

//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//---------END--END---------  Initialisation and Memory Handling ------END--END----------
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++


//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//--------------------------  GUI Elements and Delegates --------------------------------
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

- (void)updateGui {
    NSString* northSuffix;
    double actualDegreesAverrageX = 0;
    double actualDegreesAverrageY = 0;
    double actualDegreesAverrage = 0;
    
    [smoothDegrees addObject:[NSNumber numberWithDouble:actualDegrees]];
    if ([smoothDegrees count] > 20)
    {
        [smoothDegrees removeObjectAtIndex:0];
        for (int x = 0; x < [smoothDegrees count]; x++)
        {
            actualDegreesAverrageX += cos([[smoothDegrees objectAtIndex:x]doubleValue] * radianConst);
            actualDegreesAverrageY += sin([[smoothDegrees objectAtIndex:x]doubleValue] * radianConst);
        }
        actualDegreesAverrageX = actualDegreesAverrageX/[smoothDegrees count];
        actualDegreesAverrageY = actualDegreesAverrageY/[smoothDegrees count];
        actualDegreesAverrage = atan2(actualDegreesAverrageY, actualDegreesAverrageX) * (180/M_PI);
        if(deviceOrientation == UIInterfaceOrientationPortraitUpsideDown)
            actualDegreesAverrage -= 180;
        actualDegreesAverrage = actualDegreesAverrage < 0 ? actualDegreesAverrage + 360 : actualDegreesAverrage;
    }
    
    
    int degreesRounded = (int)(actualDegreesAverrage + (actualDegreesAverrage>0 ? 0.5 : -0.5));
    
    
    degreesRounded = degreesRounded == 360 ? 0 : degreesRounded; //Kill the 360 degrees
    
	if ((degreesRounded >= 337.5) || (degreesRounded <= 22.5)) 
    {
		northSuffix = @"N";	
    }
    else if ((degreesRounded > 22.5) && (degreesRounded <= 67.5)) 
    {
        northSuffix = @"NE";	
    }
    else if ((degreesRounded > 67.5) && (degreesRounded <= 112.5)) 
    {
        northSuffix = @"E";	
    }
    else if ((degreesRounded > 112.5) && (degreesRounded <= 157.5)) 
    {
        northSuffix = @"SE";	
    }
    else if ((degreesRounded > 157.5) && (degreesRounded <= 202.5)) 
    {
        northSuffix = @"S";	
    }
    else if ((degreesRounded > 202.5) && (degreesRounded <= 247.5)) 
    {
        northSuffix = @"SW";	
    }
    else if ((degreesRounded > 247.5) && (degreesRounded <= 292.5)) 
    {
        northSuffix = @"W";	
    }
    else if ((degreesRounded > 292.5) && (degreesRounded <= 337.5)) 
    {
        northSuffix = @"NW";	
    }
    
    labelDegrees.text = [NSString stringWithFormat:@"%d° %@", degreesRounded, northSuffix];

    
    CGAffineTransform mapTransfom = CGAffineTransformMakeRotation(-1 * actualDegreesAverrage * radianConst);
    
    //Cool Annimation
    //[UIView beginAnimations:nil context:NULL];
    //[UIView setAnimationBeginsFromCurrentState:YES];
    //[UIView setAnimationDuration:0.025f];
    if(!stopGuiUpdate)
        [viewWindrose setTransform:mapTransfom];
    //[UIView commitAnimations];
}

-(void)openAbout {
    if ([[UIDevice currentDevice] userInterfaceIdiom] == UIUserInterfaceIdiomPhone) {
        aboutView = [[AboutView alloc] initWithNibName:@"AboutView_iPhone" bundle:[NSBundle mainBundle]];
    } else {
        aboutView = [[AboutView alloc] initWithNibName:@"AboutView_iPad" bundle:[NSBundle mainBundle]];
    }

    [aboutView setModalTransitionStyle:UIModalTransitionStyleCoverVertical];
    [self presentModalViewController:aboutView animated:YES];
    [aboutView release];
}

-(void)updateGuiLocation {
    if(!stopGuiUpdate)
    {    
        NSString *format = [locationFormats objectAtIndex:locationFormatPointer];
        NSString *heightunit=@"ft";
        NSString *speedunit=@"mph";
        
        if(tempLocation == nil)
            return;
        
        ExtendedLocation* myExLoc = [[[ExtendedLocation alloc] initWithLocation:tempLocation] autorelease];
        
        if ([format isEqualToString:@"Feature"]) {
            //Get right Metric format
            if([(NSNumber *)[[NSLocale currentLocale] objectForKey:NSLocaleUsesMetricSystem] boolValue])
            {
                heightunit = @"m";
                speedunit = @"kmh";
            }
            int height = [myExLoc getAltitude:heightunit];
            double speed = [myExLoc getSpeed:speedunit];
            
            speedunit = [speedunit isEqualToString:@"kmh"] == true ? @"km/h": @"MPH";
            
            [labelLat setText:[NSString stringWithFormat:@"%d %@",height, heightunit]];
            [labelLong setText:[NSString stringWithFormat:@"%.1f %@",speed, speedunit]];
        }
        else    {
            LocationCoordinate2DConverted myLoc = [myExLoc getLocationwithFormat:format];
            
            [labelLat setText:myLoc.latitudeWithFormat];
            [labelLong setText:myLoc.longitudeWithFormat];
        }

    }
}

-(void)handleSwipeFrom:(UISwipeGestureRecognizer *)recognizer {
    if([recognizer direction] == UISwipeGestureRecognizerDirectionRight)
    {
        locationFormatPointer++;
        [self fadeInLabel];
    }
    if([recognizer direction] == UISwipeGestureRecognizerDirectionLeft)
    {
        locationFormatPointer--;
        [self fadeInLabel];
    }
    if([recognizer direction] == UISwipeGestureRecognizerDirectionDown)
    {
        [self openAbout];
    }
}

- (void)fadeInLabel {
    int arrayMax = [locationFormats count];
    int arrayLast = [locationFormats count] - 1;
    stopGuiUpdate = true;
    //CheckPointer
    locationFormatPointer = locationFormatPointer == arrayMax ? 0: locationFormatPointer;
    locationFormatPointer = locationFormatPointer == -1 ? arrayLast:locationFormatPointer;
    [[NSUserDefaults standardUserDefaults] setInteger:locationFormatPointer forKey:@"LocationFormat"];
    
    [UIView beginAnimations:nil context:NULL];
    [UIView setAnimationDelegate:self];
    [UIView setAnimationDidStopSelector:@selector(animationDidStop:finished:context:)];
    [UIView setAnimationDuration:0.5];
    [labelLat setAlpha:0];
    [labelLong setAlpha:0];
    [UIView commitAnimations];
}

//This delegate is called after the completion of Animation.
-(void)animationDidStop:(NSString *)animationID finished:(NSNumber *)finished context:(void *)context
{
    stopGuiUpdate = false;
    [self updateGuiLocation];
    [UIView beginAnimations:nil context:NULL];
    [UIView setAnimationDuration:0.5];
    [labelLat setAlpha:1];
    [labelLong setAlpha:1];
    [UIView commitAnimations];
}

- (void)showHudonView:(NSString *)label WithDetailLabel:(NSString *)detailLabel andImage:(NSString *)image{
    HUD = nil;
    HUD = [[MBProgressHUD alloc] initWithView:self.view];
	[self.view addSubview:HUD];
	HUD.customView = [[[UIImageView alloc] initWithImage:[UIImage imageNamed:image]] autorelease];
	HUD.mode = MBProgressHUDModeCustomView;
	
	HUD.delegate = self;
	HUD.labelText = label;
	HUD.detailsLabelText = detailLabel;
	
	[HUD show:YES];
	[HUD hide:YES afterDelay:2];
}

- (IBAction)buttonBusybytes:(id)sender {
    //[self openAbout];
    [self loadBlackSkin];
}

- (IBAction)buttonNorth:(id)sender {
    [CLController.locMgr stopUpdatingHeading];
    trueNorth = trueNorth == false ? true : false;
    [[NSUserDefaults standardUserDefaults] setBool:trueNorth forKey:@"NorthFormat"];
    [[NSUserDefaults standardUserDefaults] synchronize];
    
    if(trueNorth)
        [north setImage:[UIImage imageNamed: @"northArrowRed.png"] forState:UIControlStateNormal];
    else
        [north setImage:[UIImage imageNamed: @"northArrowBlue.png"] forState:UIControlStateNormal];
    
    [CLController.locMgr startUpdatingHeading];
}

- (void)tabAction:(UITapGestureRecognizer *)recognizer {
    
    if(tempLocation == nil)
    {
         [self showHudonView:@"Copied to Clipboard FAILD" WithDetailLabel:@"Please Check GPS Signal" andImage:@"hudIconFail.png"];   
        return;
    }
    
    NSString *format = [locationFormats objectAtIndex:locationFormatPointer];
    if ([format isEqualToString:@"Feature"]) {
        format = @"DegDec";
    }
    
    ExtendedLocation* myLoc = [[[ExtendedLocation alloc] initWithLocation:tempLocation] autorelease];
    LocationCoordinate2DConverted myLoc2D = [myLoc getLocationwithFormat:format];
    
    NSMutableString *body = [NSMutableString string];
    [body appendString:[NSString stringWithFormat:@"Latitude: %@\n", myLoc2D.latitudeWithFormat]];
    [body appendString:[NSString stringWithFormat:@"Longitude: %@\n", myLoc2D.longitudeWithFormat]];
    [body appendString:[NSString stringWithFormat:@"Course: %@\n", labelDegrees.text]];
    
    pasteboard.string = body;

    
    [self showHudonView:@"Copied to Clipboard" WithDetailLabel:@"" andImage:@"hudIconSuccess.png"];
}

//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//---------END--END---------  GUI Elements and Delegates ------------END--END----------------
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++


//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//--------------------------  Location Handling and Delegates -------------------------------
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

- (void)locationUpdate:(ExtendedLocation *)location {
    tempLocation = [[location CurrentLocation] copy];
    [self updateGuiLocation];
}

- (void)locationError:(NSError *)error {
    
}

- (void)locationHeadingUpdate:(CLHeading*)newHeading {
    if(trueNorth)
        actualDegrees = newHeading.trueHeading;
    else
        actualDegrees = newHeading.magneticHeading;

}


//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//---------END--END---------  Location Handling and Delegates ---------END--END-----------------
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

- (void)dealloc {
    [labelDegrees release];
    [viewWindrose release];
    [labelLat release];
    [labelLong release];
    [north release];
    [imgMainBG release];
    [imgBigArrow release];
    [imgCompassMain release];
    [imgCompassWindrose release];
    [imgCompassGlass release];
    [busybytes release];
    [super dealloc];
}

@end
